#ifndef CAM_DATA_H
#define CAM_DATA_H

#include <cstdint>

namespace Localization
{
namespace PointLoc
{

struct Vector2f {
  float x;
  float y;
};

struct Vector3f {
  float x;
  float y;
  float z;
};

static constexpr std::uint8_t kMaxLaneMarkerNum{4U};
static constexpr std::uint8_t kMaxTrafficLightNum{8U};
static constexpr std::uint8_t kMaxTrafficSignNum{8U};
static constexpr std::uint8_t kMaxObstacleNum{10U};

enum class LaneMeasuringStatus : std::uint8_t {
  UNKNOWN = 0,   // Unknown
  NEW = 1,       // New
  MEASURED = 2,  // Measured
  PREDICTED = 3, // Predicted
  NOT_VALID = 4  // Not valid
};

enum class LaneTypeClass : std::uint8_t {
  UNKNOWN = 0,                 // Unknown
  SOLID = 1,                   // Solid
  ROAD_EDGE = 2,               // Road edge
  DASHED = 3,                  // Dashed
  LEFT_DASHED_RIGHT_SOLID = 4, // Left dashed right solid
  LEFT_SOLID_RIGHT_DASHED = 5, // Left solid right dashed
  DOUBLE_DASHED = 6,           // Double dashed
  DOUBLE_SOLID = 7,            // Double solid
  BARRIER = 8,                 // Barrier
  INVALID = 9                  // Invalid
};

enum class LaneMarkerColor : std::uint8_t {
  UNKNOWN = 0,           // Unknown
  WHITE = 1,             // White
  YELLOW_ORANGE_RED = 2, // Yellow orange red
  BLUE_GREEN = 3         // Blue green
};

enum class LaneQuality : std::uint8_t {
  UNKNOWN = 0,  // Unknown
  LOW = 1,      // Low
  HIGH = 2,     // High
  VERY_HIGH = 3 // Very high
};

enum class RoadType : std::uint8_t {
  UNKNOWN = 0, // Unknown
  HIGHWAY = 1, // Highway
  URBAN = 2,   // Urban
  RURAL = 3    // Rural
};

enum class Weather : std::uint8_t {
  UNKNOWN = 0,           // Unknown
  SUNNY_CLOUDY = 1,      // Sunny cloudy
  RAINY_HEAVY_RAINY = 2, // Rainy heavy rainy
  SNOW = 3,              // Snow
  FOGGY = 4              // Foggy
};

enum class TrafficLightColor : std::uint8_t {
  UNKNOWN = 0, // Unknown
  RED = 1,     // Red
  GREEN = 2,   // Green
  YELLOW = 3,  // Yellow
  OTHER = 4,   // Other
  OFF = 5      // Off
};

enum class TrafficLightShapeType : std::uint8_t {
  UNKNOWN = 0,   // Unknown
  VERTICAL = 1,  // Vertical
  HORIZONTAL = 2 // Horizontal
};

enum class TrafficLightClassificationType : std::uint8_t {
  UNKNOWN = 0,             // Unknown
  CIRCLE = 1,              // Vertical
  FORWARD = 2,             // Horizontal
  LEFT = 3,                // Left
  RIGHT = 4,               // Right
  RETURN = 5,              // Return
  PEDESTRIAN = 6,          // Pedestrian
  NON_MOTOR = 7,           // Motor not allowed
  TIME = 8,                // Time counter
  OTHER = 9,               // Others
  LEFT_AND_RETURN = 10,    // Left and return
  FORWARD_AND_LEFT = 11,   // Forward and left
  FORWARD_AND_RIGHT = 12,  // Forward and right
  NO_DRIVE_INTO = 13,      // Not allowed
  ALLOW_DRIVE_INTO = 14,   // Allowed
  TEXT_OF_ALLOW_PED = 15,  // Text of allow pedestrian
  SIGN_OF_ALLOW_PED = 16,  // Sign of allow pedestrian
  TEXT_OF_FORBID_PED = 17, // Text of forbid pedestrian
  SIGN_OF_FORBID_PED = 18  // Sign of forbid pedestrian
};

enum class TrafficSignType : std::uint8_t {
  CONSTRUCTION = 0,                 // Construction
  MIN_SPEED_LIMIT_100 = 1,          // Min 100
  MIN_SPEED_LIMIT_110 = 2,          // Min 110
  MIN_SPEED_LIMIT_50 = 3,           // Min 50
  MIN_SPEED_LIMIT_60 = 4,           // Min 60
  MIN_SPEED_LIMIT_70 = 5,           // Min 70
  MIN_SPEED_LIMIT_80 = 6,           // Min 80
  MIN_SPEED_LIMIT_90 = 7,           // Min 90
  NO_PASSING_REV = 8,               // No passing rev
  OTHER_CONSTRUCTION_SIGN = 9,      // Other construction sign
  OTHER_DETFP = 10,                 // Other DETFP
  OTHER_EXIT = 11,                  // Other exit
  OTHER_GUIDESIGN_BLUEWHITE = 12,   // Other blue and white guide sign
  OTHER_GUIDESIGN_GREENWHITE = 13,  // Other green and white guide sign
  OTHER_GUIDESIGN_WHITEBLACK = 14,  // Other black and white guide sign
  OTHER_INDICATIONS = 15,           // Other indications
  OTHER_LIGHTARROW = 16,            // Other light arrow
  OTHER_MIRROR = 17,                // Other mirror
  OTHER_NOPARKING = 18,             // Other no parking
  OTHER_PROHIBIT = 19,              // Other prohibit
  OTHER_REDCIRCLELIGHT = 20,        // Other reid circle light
  OTHER_ROUTEID_REDWHITE = 21,      // Other red and while route id
  OTHER_ROUTEID_YELLOWBLACK = 22,   // Other yellow and black route id
  OTHER_TRAVELSIGN_BROWNWHITE = 23, // Other brown and white travel sign
  OTHER_WARNING = 24,               // Other warning
  SPEED_LIMIT_10 = 25,              // Max 10
  SPEED_LIMIT_100 = 26,             // Max 100
  SPEED_LIMIT_100_ELECTRIC = 27,    // Max 100 electric
  SPEED_LIMIT_110 = 28,             // Max 110
  SPEED_LIMIT_110_ELECTRIC = 29,    // Max 110 electric
  SPEED_LIMIT_120 = 30,             // Max 120
  SPEED_LIMIT_120_ELECTRIC = 31,    // Max 120 electric
  SPEED_LIMIT_15 = 32,              // Max 15
  SPEED_LIMIT_20 = 33,              // Max 20
  SPEED_LIMIT_25 = 34,              // Max 25
  SPEED_LIMIT_30 = 35,              // Max 30
  SPEED_LIMIT_30_ELECTRIC = 36,     // Max 30 electric
  SPEED_LIMIT_35 = 37,              // Max 35
  SPEED_LIMIT_40 = 38,              // Max 40
  SPEED_LIMIT_40_ELECTRIC = 39,     // Max 40 electric
  SPEED_LIMIT_5 = 40,               // Max 5
  SPEED_LIMIT_50 = 41,              // Max 50
  SPEED_LIMIT_50_ELECTRIC = 42,     // Max 50 electric
  SPEED_LIMIT_55 = 43,              // Max 55
  SPEED_LIMIT_5_ELECTRIC = 44,      // Max 5 electric
  SPEED_LIMIT_60 = 45,              // Max 60
  SPEED_LIMIT_60_ELECTRIC = 46,     // Max 60 electric
  SPEED_LIMIT_65 = 47,              // Max 65
  SPEED_LIMIT_70 = 48,              // Max 70
  SPEED_LIMIT_70_ELECTRIC = 49,     // Max 70 electric
  SPEED_LIMIT_80 = 50,              // Max 80
  SPEED_LIMIT_80_ELECTRIC = 51,     // Max 80 electric
  SPEED_LIMIT_90 = 52,              // Max 90
  SPEED_LIMIT_90_ELECTRIC = 53,     // Max 90 electric
  SPEED_LIMIT_REV = 54,             // Speed limit rev
  DEFAULT = 255                     // Default
};

enum class TrafficSignRelevancy : std::uint8_t {
  UNDEFINED = 0,       // Undefined
  IN_HOSTLANE = 1,     // In host lane
  NOT_IN_HOSTLANE = 2, // Not in host lane
  HIGHWAY_EXIT = 3,    // Highway exit
  SIGN_ON_TURN = 4,    // Sign on turn
  FAR_IRRELEVANT = 5   // Far irrelevant
};

enum class TrafficSignFilterType : std::uint8_t {
  NOT_FILTERED = 0, // Not filtered
  IRRELEVANT = 1,   // Irrelevant
  ON_VEHICLE = 2,   // On vehicle
  EMBEDDED = 3,     // Embedded
  RESERVED1 = 4,    // Reserved
  RESERVED2 = 5,    // Reserved
  RESERVED3 = 6,    // Reserved
  RESERVED4 = 7,    // Reserved
};

enum class CameraId : std::uint8_t {
  DEFAULT = 0,             // Default
  FISHEYE_BACK = 1,        // Fisheye back
  FISHEYE_FRONT = 2,       // Fisheye front
  FISHEYE_LEFT = 3,        // Fisheye left
  FISHEYE_RIGHT = 4,       // Fisheye right
  NARROW_BACK = 5,         // Narrow back
  NARROW_BACK_LEFT = 6,    // Narrow back left
  NARROW_LEFT = 7,         // Narrow left
  NARROW_FRONT_LEFT = 8,   // Narrow front left
  NARROW_FRONT = 9,        // Narrow front
  NARROW_FRONT_RIGHT = 10, // Narrow front right
  NARROW_RIGHT = 11,       // Narrow right
  NARROW_BACK_RIGHT = 12   // Narrow back right
};

enum class ObstacleValidity : std::uint8_t {
  INVALID = 0,    // Invalid
  NEW_VALID = 1,  // Detected this frame
  OLDER_VALID = 2 // Detected in previous frames
};

enum class ObstacleMotionCategory : std::uint8_t {
  INVALID = 0,      // Invalid
  UNDEFINED = 1,    // Undefined
  PASSING = 2,      // Passing
  PASSING_IN = 3,   // Passing in
  PASSING_OUT = 4,  // Passing out
  CLOSE_CUT_IN = 5, // Close cut in
  MOVING_IN = 6,    // Moving in
  MOVING_OUT = 7,   // Moving out
  CROSSING = 8,     // Crossing
  LTAP = 9,         // LTAP
  RTAP = 10,        // RTAP
  MOVING = 11,      // Moving
  PRECEEDING = 12,  // Preceeding
  ONCOMING = 13     // Oncoming
};

enum class ObstacleMotionStatus : std::uint8_t {
  INVALID = 0,      // Invalid
  UNKNOWN = 1,      // Unknown
  MOVING = 2,       // Moving
  STATIONARY = 3,   // Stationary
  STOPPED = 4,      // Stopped
  MOVING_SLOWLY = 5 // Moving slowly
};

enum class ObstacleMeasuringStatus : std::uint8_t {
  INVALID = 0,  // Invalid
  MEASURED = 1, // Measured
  PREDICTED = 2 // Predicted
};

enum class ObstacleClass : std::uint8_t {
  INVALID = 0,       // Invalid
  VEHICLE = 1,       // Vehicle
  PEDESTRIAN = 2,    // Pedestrian
  CYCLIST = 3,       // Cyclist
  GENERAL_OBJECT = 4 // General object
};

enum class VehicleSubType : std::uint8_t {
  UNKNOWN = 0,          // Unknown
  BUS = 1,              // Bus
  SMALL_MEDIUM_CAR = 2, // Small medium car
  TRUCK = 3,            // Truck
  MOTOR = 4,            // Motor
  SPECIAL_VEHICLE = 5,  // Special vehicle
  TINY_CAR = 6,         // Tiny car
  LORRY = 7             // Lorry
};

enum class PedestrianSubType : std::uint8_t {
  UNKNOWN = 0, // Unknown
  ADULT = 1,   // Adult
  CHILD = 2    // Child
};

enum class PedestrianPose : std::uint8_t {
  UNKNOWN = 0,    // Unknown
  BENDED = 1,     // Bended
  CYCLIST = 2,    // Cyclist
  LIER = 3,       // Lier
  PEDESTRIAN = 4, // Pedestrian
  SITTER = 5      // Sitter
};

enum class PedestrianOrientation : std::uint8_t {
  UNKNOWN = 0,       // Unknown
  BACK = 1,          // Back
  FRONT = 2,         // Front
  LEFT = 3,          // Left
  LEFT_ANTERIOR = 4, // Left anterior
  LEFT_BACK = 5,     // Left back
  RIGHT = 6,         // Right
  RIGHT_FRONT = 7,   // Right front
  RIGHT_BACK = 8     // Right back
};

enum class ObstacleLaneAssignment : std::uint8_t {
  UNKNOWN = 0,    // Unknown
  LEFT_LEFT = 1,  // Left left
  LEFT = 2,       // Left
  HOST = 3,       // Host lane
  RIGHT = 4,      // Right
  RIGHT_RIGHT = 5 // Right right
};

enum class ObstacleBlinkerInfo : std::uint8_t {
  UNAVAILABLE = 0, // Unknown
  OFF = 1,         // Off
  LEFT = 2,        // Left
  RIGHT = 3,       // Right
  BOTH = 4         // Both
};

// Sync message
struct alignas(8) SyncMessage {
  /// @brief Image timestamp
  std::uint64_t image_timestamp;
  /// @brief SPI timestamp
  std::uint64_t spi_timestamp;
  /// @brief Frame index
  std::uint32_t frame_index;
  /// @brief Camera id
  CameraId camera_id;
  /// @brief Protocol major version
  std::uint8_t protocol_major_version;
  /// @brief Protocol minor version
  std::uint8_t protocol_minor_version;
  std::uint8_t padding;
};

// Lane marker
struct alignas(8) LaneMarker {
  /// @brief Track ID
  std::uint16_t track_id;
  /// @brief Confidence
  std::uint8_t confidence;
  /// @brief Measuring status
  LaneMeasuringStatus measuring_status;
  /// @brief Type class (reserved)
  LaneTypeClass type_class;
  /// @brief Color
  LaneMarkerColor color;
  /// @brief Quality
  LaneQuality quality;
  /// @brief Whether is ego car is crossing lane
  bool is_lane_crossing;
  /// @brief Width (m)
  float width;
  /// @brief View range start (m)
  float start;
  /// @brief View range end (m)
  float end;
  /// @brief C0 (m)
  float c0;
  /// @brief C1 (rad)
  float c1;
  /// @brief C2 (1/m)
  float c2;
  /// @brief C3 (1/m/m)
  float c3;
  /// @brief Time for ego car to cross lane
  float time_to_lane_crossing;
};

// Road info
struct alignas(8) RoadInfo {
  /// @brief Road type
  RoadType road_type;
  /// @brief Current weather
  Weather current_weather;
  /// @brief Protocol major version
  std::uint8_t protocol_major_version;
  /// @brief Protocol minor version
  std::uint8_t protocol_minor_version;
  /// @brief Host estimated width
  float host_estimated_width;
};

// Traffic light header
struct alignas(8) TrafficLightHeader {
  /// @brief Object number
  std::uint8_t num_objects;
  /// @brief Man object id (reserved)
  std::uint8_t main_object_id;
  /// @brief Protocol major version
  std::uint8_t protocol_major_version;
  /// @brief Protocol minor version
  std::uint8_t protocol_minor_version;
  std::uint8_t padding[4];
};

// Traffic light
struct alignas(8) TrafficLight {
  /// @brief Internal id
  std::uint32_t internal_id;
  /// @brief Id
  std::uint8_t id;
  /// @brief Confidence
  std::uint8_t confidence;
  /// @brief Traffic light color
  TrafficLightColor color;
  /// @brief Shape type (reserved)
  TrafficLightShapeType shape_type;
  /// @brief Classification type
  TrafficLightClassificationType classification_type;
  /// @brief Whether is blinking (reserved)
  bool is_blinking;
  std::uint8_t padding_1[2];
  /// @brief Width (reserved)
  float width;
  /// @brief Position (m)
  Vector3f position;
  std::uint8_t padding_2[4];
};

// Traffic sign header
struct alignas(8) TrafficSignHeader {
  /// @brief Object number
  std::uint8_t num_objects;
  /// @brief Protocol major version
  std::uint8_t protocol_major_version;
  /// @brief Protocol minor version
  std::uint8_t protocol_minor_version;
  std::uint8_t padding[5];
};

// Traffic sign
struct alignas(8) TrafficSign {
  /// @brief Internal id
  std::uint32_t internal_id;
  /// @brief Id
  std::uint8_t id;
  /// @brief Confidence
  std::uint8_t confidence;
  /// @brief Type
  TrafficSignType type;
  /// @brief Road mark type (reserved)
  std::uint8_t road_mark_type;
  /// @brief Relevancy (reserved)
  TrafficSignRelevancy relevancy;
  /// @brief Filter type (reserved)
  TrafficSignFilterType filter_type;
  /// @brief Whether is approved
  bool is_approved;
  std::uint8_t padding;
  /// @brief Position (m)
  Vector3f position;
};

// Obstacle header
struct alignas(8) ObstacleHeader {
  /// @brief Object number
  std::uint8_t num_objects;
  /// @brief CIPV id
  std::uint8_t cipv_id;
  /// @brief MCP id
  std::uint8_t mcp_id;
  /// @brief Whether CIPV TTC is valid
  bool is_cipv_ttc_valid;
  /// @brief Whether MCP TTC is valid
  bool is_mcp_ttc_valid;
  /// @brief Protocol major version
  std::uint8_t protocol_major_version;
  /// @brief Protocol minor version
  std::uint8_t protocol_minor_version;
  std::uint8_t padding;
  /// @brief CIPV TTC
  float cipv_ttc;
  /// @brief MCP TTC
  float mcp_ttc;
};

// Obstacle
struct alignas(8) Obstacle {
  /// @brief Internal id
  std::uint32_t internal_id;
  /// @brief Id
  std::uint8_t id;
  /// @brief Validity
  ObstacleValidity validity;
  /// @brief Whether is replaced in this frame (reserved)
  bool is_replaced;
  /// @brief Confidence
  std::uint8_t confidence;
  /// @brief Frame age
  std::uint16_t age;
  /// @brief Motion category
  ObstacleMotionCategory motion_category;
  /// @brief Motion status
  ObstacleMotionStatus motion_status;
  /// @brief Measuring status
  ObstacleMeasuringStatus measuring_status;
  /// @brief Class
  ObstacleClass obstacle_class;
  /// @brief Vehicle sub type
  VehicleSubType vehicle_sub_type;
  /// @brief Pedestrian sub type
  PedestrianSubType pedestrian_sub_type;
  /// @brief Pedestrian pose
  PedestrianPose pedestrian_pose;
  /// @brief Pedestrian orientation
  PedestrianOrientation pedestrian_orientation;
  /// @brief Lane assignment
  ObstacleLaneAssignment lane_assignment;
  /// @brief Blinker info
  ObstacleBlinkerInfo blinker_info;
  /// @brief Length (m)
  float length;
  /// @brief Width (m)
  float width;
  /// @brief Height (m)
  float height;
  /// @brief Whether is CIPV
  bool is_cipv;
  /// @brief Whether is MCP
  bool is_mcp;
  /// @brief Whether brake is on
  bool is_brake_on;
  std::uint8_t padding;
  /// @brief Center angle (deg)
  float center_angle;
  /// @brief Relative position (longitudinal distance, lateral distance) (m)
  Vector2f position_rel;
  /// @brief Relative velocity (longitudinal, lateral) (m/s)
  Vector2f velocity_rel;
  /// @brief Relative acceleration (longitudinal, lateral) (m/s/s)
  Vector2f accleration_rel;
  /// @brief Position standard deviation (m)
  Vector2f position_std;
  /// @brief Velocity standard deviation (m/s)
  Vector2f velocity_std;
  /// @brief Absolute velocity (longitudinal, lateral) (m/s)
  Vector2f velocity_abs;
  /// @brief Angle (deg)
  float angle;
  /// @brief Angle standard deviation (deg)
  float angle_std;
};

// J2 data
struct alignas(8) J2Data {
  double stamp_;
  /// @brief Sync message
  SyncMessage sync_message;
  /// @brief Lane markers
  LaneMarker lane_markers[kMaxLaneMarkerNum];
  /// @brief Road info
  RoadInfo road_info;
  /// @brief Traffic light header
  TrafficLightHeader traffic_light_header;
  /// @brief Traffic lights
  TrafficLight traffic_lights[kMaxTrafficLightNum];
  /// @brief Traffic sign header
  TrafficSignHeader traffic_sign_header;
  /// @brief Traffic signs
  TrafficSign traffic_signs[kMaxTrafficSignNum];
  /// @brief Obstacle header
  ObstacleHeader obstacle_header;
  /// @brief Obstacles
  Obstacle obstacles[kMaxObstacleNum];
};

} // namespace PointLoc
} // namespace Localization
#endif
